package jpos.config.simple.editor;

///////////////////////////////////////////////////////////////////////////////
//
// This software is provided "AS IS".  The JavaPOS working group (including
// each of the Corporate members, contributors and individuals)  MAKES NO
// REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE,
// EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
// NON-INFRINGEMENT. The JavaPOS working group shall not be liable for
// any damages suffered as a result of using, modifying or distributing this
// software or its derivatives. Permission to use, copy, modify, and distribute
// the software and its documentation for any purpose is hereby granted. 
//
///////////////////////////////////////////////////////////////////////////////


import java.io.File;
import javax.swing.filechooser.FileFilter;

import jpos.util.JposProperties;
import jpos.loader.JposServiceLoader;

/**
 * Simple filter class to accecpt xml and cfg files
 * @author E. Michael Maximilien (maxim@us.ibm.com)
 * @since 1.3 (SF 2K meeting)
 */
class JposEntryEditorFileFilter extends FileFilter
{
    /**
     * Tests whether or not the specified abstract pathname should be
     * included in a pathname list.
     * @param  pathname  The abstract pathname to be tested
     * @return  <code>true</code> if and only if <code>pathname</code>
     *          should be included
     */
    public boolean accept( File pathname ) 
	{
        String filename = pathname.getName();
        
        if( pathname.isDirectory() )
            return true;

        if( jposProperties.isPropertyDefined( JposProperties.JPOS_REG_POPULATOR_CLASS_PROP_NAME ) )
            if( jposProperties.getPropertyString( JposProperties.JPOS_REG_POPULATOR_CLASS_PROP_NAME ).endsWith( "SimpleXmlRegPopulator" ) )
                mode = "xml";

        if( mode.equalsIgnoreCase( "xml" ) )
			return filename.endsWith( ".xml" );
        else
            return filename.endsWith( ".cfg" );
    }

	/** @return the description for this FileFilter */
    public String getDescription() 
	{
        return ( mode.equalsIgnoreCase( "xml" ) ? XML_ENTRIES_STRING : SERIALIZED_ENTRIES_STRING );
    }

    //---------------------------------------------------------------------
    // Instance variables
    //

    private static String mode = "xml";

    private JposProperties jposProperties = JposServiceLoader.getManager().getProperties();

    //---------------------------------------------------------------------
    // Class constants
    //

	//<i18n>
	public static final String XML_ENTRIES_STRING = "XML entries (*.xml)";
	public static final String SERIALIZED_ENTRIES_STRING = "Serialized entries (*.cfg)";
	//</i18n>
}
