package jpos.profile;

///////////////////////////////////////////////////////////////////////////////
//
// This software is provided "AS IS".  The JavaPOS working group (including
// each of the Corporate members, contributors and individuals)  MAKES NO
// REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE,
// EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
// NON-INFRINGEMENT. The JavaPOS working group shall not be liable for
// any damages suffered as a result of using, modifying or distributing this
// software or its derivatives. Permission to use, copy, modify, and distribute
// the software and its documentation for any purpose is hereby granted. 
//
///////////////////////////////////////////////////////////////////////////////

/**
 * Defines a simple list of PropValue objects
 * @since 1.3 (SF 2K meeting)
 * @author E. Michael Maximilien (maxim@us.ibm.com)
 */
public interface PropValueList
{
	/** @return the current size of this list */
	public int getSize();

	/** @return true if this list is empty */
	public boolean isEmpty();

	/**
	 * Adds a new PropValue to this list
	 * @param propValue the propValue to add
	 */
	public void add( PropValue propValue );

	/**
	 * Removes the PropValue to this list
	 * @param propValue the propValue to remove
	 */
	public void remove( PropValue propValue );

	/** Removes all PropValue in this list */
	public void removeAll();

	/**
	 * @return true if the PropValue is already in the list
	 * @param propValue the PropValue
	 */
	public boolean contains( PropValue propValue );

	/** @return an PropValueList.Iterator object for this list */
	public PropValueList.Iterator iterator();

	//-------------------------------------------------------------------------
	// Inner interfaces
	//

	/**
	 * Simple interface to iterate through the list
	 * @author E. Michael Maximilien (maxim@us.ibm.com)
	 * @since 1.3 (SF 2K meeting)
	 */
	public interface Iterator
	{ 
		/** @return the next PropValue in the iterator */
		public PropValue next();

		/** @return true if there is a next PropValue in the iterator */
		public boolean hasNext();
	}
}
